require 'persisto/exceptions'

module Persisto
	module Repository
		def self.included(base)
			base.extend ClassMethods
		end

		module ClassMethods
			def set_mapper val
				@mapper = val
			end

			def mapper
				@mapper || raise(::Persisto::RepositoryError, 'repository mapper not assigned')
			end

			def store_registry
				@store_registry ||= {}
			end

			def register_store store_name, store_class
				store_registry[store_name] = store_class
				define_method(store_name){ store(store_name) }
			end
		end

		def initialize adaptor
			@adaptor = adaptor
			@stores = {}
		end

		def transaction &block
			adaptor.transaction &block
		end

		private

		def store store_name
			@stores[store_name] ||= self.class.store_registry[store_name].new(adaptor, self.class.mapper.new)
		end

		def adaptor
			@adaptor
		end
	end
end