require 'persisto/exceptions'

module Persisto
	module Store
		def self.included(base)
			base.extend ClassMethods
		end

		module ClassMethods
			def set_table_name val
				@table_name = val
			end

			def table_name
				@table_name || raise(StoreError, 'table name not assigned')
			end
		end

		attr_reader :adaptor, :mapper

		def initialize adaptor, mapper
			@adaptor = adaptor
			@mapper = mapper
		end

		def all query_object
			assign_to_query_object(query_object)
			parse(adaptor.select(query_object))
		end

		def first query_object
			assign_to_query_object(query_object)
			parse(adaptor.select(query_object)).first
		end

		def first! query_object
			assign_to_query_object(query_object)
			parse(adaptor.select(query_object)).first || raise(EntityNotFound, "#{query_object.class.name}: entity not found")
		end

		def insert query_object
			assign_to_query_object(query_object)
			adaptor.insert(query_object)
		end

		def update query_object
			assign_to_query_object(query_object)
			adaptor.update(query_object)
		end

		def delete query_object
			assign_to_query_object(query_object)
			adaptor.delete(query_object)
		end

		def transaction &block
			adaptor.transaction &block
		end

		def dump entity
			mapper.dump_store(self.class.table_name, entity)
		end

		def parse data
			[*data].map{|d| mapper.parse(d)}
		end

		private

		def assign_to_query_object query_object
			query_object.store = self
		end
	end
end